use std::str::FromStr as _;

use crate::color::{self, EscapeSequence as _};

struct FugaLogger {
    level: log::LevelFilter,
}

impl log::Log for FugaLogger {
    fn enabled(&self, metadata: &log::Metadata<'_>) -> bool {
        metadata.level() <= self.level
    }

    fn log(&self, record: &log::Record<'_>) {
        if self.enabled(record.metadata()) {
            let style = match record.level() {
                log::Level::Error => color::RED.to_escape_sequence(),
                log::Level::Warn => color::ORANGE.to_escape_sequence(),
                log::Level::Info => color::WHITE.to_escape_sequence(),
                log::Level::Debug => color::BLUE.to_escape_sequence(),
                log::Level::Trace => color::PURPLE.to_escape_sequence(),
            };

            eprintln!(
                "{style}{: <5}{} {}",
                record.level(),
                color::Reset.render(),
                record.args()
            );
        }
    }

    fn flush(&self) {}
}

pub fn init() -> Result<(), log::SetLoggerError> {
    let level = if let Some(level) = std::option_env!("FUGA_LOG") {
        log::LevelFilter::from_str(level).unwrap_or(log::LevelFilter::Off)
    } else {
        log::LevelFilter::Info
    };

    let logger = FugaLogger { level };
    log::set_boxed_logger(Box::new(logger)).map(|()| log::set_max_level(level))?;

    Ok(())
}
